program Project1;

const
  CPU_Clock = 16000000; // Taktfrequenz Arduino, default 16MHz.
  F_SCL = 100000;       // SCL frequency
  Baud = 9600;          // Baudrate
  UCSZ01 = 2;           // Gibt es nicht in der Unit Atmega328p.

  procedure sei; assembler; inline;
  asm
           Sei
  end;

  procedure cli; assembler; inline;
  asm
           Cli
  end;

  procedure mysleep(t: int32);
  var
    i: Int32;
  begin
    for i := 0 to t do begin
      asm
               Nop;
      end;
    end;
  end;

  procedure UARTInit;
  const
    Teiler = CPU_Clock div (2 * Baud) - 1;
  begin
    UBRR0 := Teiler;
    UCSR0A := UCSR0A or (0 shl U2X0);
    UCSR0B := UCSR0B or (1 shl TXEN0) or (1 shl RXEN0) {or (1 shl RXCIE0)};
    UCSR0C := UCSR0C or (1 shl UMSEL0) or (1 shl UCSZ01) or (1 shl UCSZ0);

    DDRD := DDRD or (1 shl 4);
  end;

  procedure UARTSendChar(c: char);
  begin
    while UCSR0A and (1 shl UDRE0) = 0 do begin
    end;
    UDR0 := byte(c);
  end;

  function UARTReadChar: char;
  begin
    while UCSR0A and (1 shl RXC0) = 0 do begin
    end;
    Result := char(UDR0);
  end;

  procedure UARTSendString(s: ShortString);
  var
    i: integer;
  begin
    for i := 1 to length(s) do begin
      UARTSendChar(s[i]);
    end;
  end;

  // === I2C

var
  Data: integer;
  Buf: array[0..3] of byte;
  s: shortstring;


  procedure TWIInit;
  const
    Prescaler = 1;
    //    TWBR_val = ((((CPU_Clock div F_SCL) div Prescaler) - 16) div 2);
    TWBR_val = (((CPU_Clock div F_SCL) - 16) div 2);
  begin
    TWSR := 0;
    TWBR := byte(TWBR_val);
    TWCR := (1 shl TWEN);
  end;

  procedure TWIStart(addr: byte);
  begin
    TWCR := 0;
    TWCR := (1 shl TWINT) or (1 shl TWSTA) or (1 shl TWEN);
    while ((TWCR and (1 shl TWINT)) = 0) do begin
    end;

    TWDR := addr;
    TWCR := (1 shl TWINT) or (1 shl TWEN);
    while ((TWCR and (1 shl TWINT)) = 0) do begin
    end;
  end;

  procedure TWIStop;
  begin
    TWCR := (1 shl TWINT) or (1 shl TWSTO) or (1 shl TWEN);
  end;

  procedure TWIWrite(u8data: byte);
  begin
    TWDR := u8data;
    TWCR := (1 shl TWINT) or (1 shl TWEN);
    while ((TWCR and (1 shl TWINT)) = 0) do begin
    end;
  end;

  function TWIReadACK: byte;
  begin
    TWCR := (1 shl TWINT) or (1 shl TWEN) or (1 shl TWEA);
    while (TWCR and (1 shl TWINT)) = 0 do begin
    end;
    Result := TWDR;
  end;

  function TWIReadNACK: byte;
  begin
    TWCR := (1 shl TWINT) or (1 shl TWEN);
    while (TWCR and (1 shl TWINT)) = 0 do begin
    end;
    Result := TWDR;
  end;

  // === Ende I2C

  procedure WriteADS1115(addr: UInt16);
  begin
    Buf[0] := 1;
    Buf[1] := %11100011;
    Buf[2] := %11000011;
    TWIStart($48);
    TWIWrite(Buf[0]);
    TWIWrite(Buf[1]);
    TWIWrite(Buf[2]);
    TWIStop;
  end;

  function ReadADS1115(addr: UInt16): UInt16;
  begin
    Buf[0] := 0;
    Buf[1] := 0;
    Buf[2] := 0;

    TWIStart($48);
    TWIWrite(Buf[0]);
    TWIStop;

    TWIStart($48 or 1);       // ???
    Buf[0] := TWIReadACK;     // Hier hängt er !
    Buf[1] := TWIReadNACK;
    TWIStop;

    Result := Buf[0] * $100 + Buf[1];
  end;

begin
  cli;
  Buf[0] := 1;
  Buf[1] := %11100011;
  Buf[2] := %11000011;
  UARTInit;
  TWIInit;
  sei;

  repeat
    WriteADS1115($48);
    Data := ReadADS1115($48);

    str(Data: 10, s);
    UARTSendString(s);
  until 1 = 2;
end.
